@@ -1,3 +1,4 @@
+from functools import reduce
 from pamda.utils import utils
 from pamda.curry_fn import curry_fn
 
@@ -135,7 +136,7 @@ class pamda_core(utils):
             - What: The value to appropriate to the end of the path
         - `data`:
             - Type: dict
-            - What: A dictionary to check if the path exists
+            - What: A dictionary in which to associate the given value to the given path
 
         Example:
 
@@ -146,14 +147,8 @@ class pamda_core(utils):
         """
         if isinstance(path, str):
             path=[path]
-        if len(path) > 1:
-            if path[0] not in data:
-                data[path[0]] = {}
-            data[path[0]] = self.assocPath(data=data[path[0]], path=path[1:],value=value)
-            return data
-        else:
-            data[path[0]] = value
-            return data
+        reduce(self.getForceDict, path[:-1], data).__setitem__(path[-1],value)
+        return data
 
     def assocPathComplex(self, default, default_fn, path, data):
         """
@@ -184,18 +179,135 @@ class pamda_core(utils):
         p.assocPathComplex(default=[2], default_fn=lambda x:x+[1], path=['a','c'], data=data) #=> {'a':{'b':1,'c':[2,1]}}
         ```
         """
-        if len(path) > 1:
-            if path[0] not in data:
-                data[path[0]] = {}
-            data[path[0]] = self.assocPathComplex(data=data[path[0]], path=path[1:], default=default, default_fn=default_fn)
-            return data
-        else:
-            if self.getArity(default_fn)!=1:
-                self.exception('`assocPathComplex` `default_fn` must be an unary (single input) function.')
-            if path[0] not in data:
-                data[path[0]] = default
-            data[path[0]] = default_fn(data[path[0]])
-            return data
+        if self.getArity(default_fn)!=1:
+            self.exception('`assocPathComplex` `default_fn` must be an unary (single input) function.')
+        if isinstance(path, str):
+            path=[path]
+        path_object = reduce(self.getForceDict, path[:-1], data)
+        path_object.__setitem__(path[-1], default_fn(path_object.get(path[-1],default)))
+        return data
+
+    def asyncRun(self, fn):
+        """
+        Function:
+
+        - Runs the supplied function asychronously
+
+        Requires:
+
+        - `fn`:
+            - Type: function | method
+            - What: The function or method to run asychronously
+            - Note: The supplied `fn` must have an arity of 0
+
+        Notes:
+
+        - To pass inputs to a function in asyncRun, first thunkify the function and pass all arguments before calling `asyncRun` on it
+        - To get the results of an `asyncRun` call `asyncWait`
+        - A thunkified function with arity of 0 can call `asyncRun` in place
+
+        Examples:
+
+        Input:
+        ```
+        import time
+
+        @p.thunkify
+        def test(name, wait):
+            print(f'{name} start')
+            time.sleep(wait)
+            print(f'{name} end')
+
+        async_test = p.asyncRun(test('a',2))
+        sync_test = test('b',1)()
+        ```
+        Output:
+        ```
+        a start
+        b start
+        b end
+        a end
+        ```
+
+
+        Input:
+        ```
+        import time
+
+        @p.thunkify
+        def test(name, wait):
+            time.sleep(wait)
+            return f"{name}: {wait}"
+
+        async_test = p.asyncRun(test('a',2))
+        print(async_test.asyncWait()) #=> a: 2
+        ```
+
+
+        Input:
+        ```
+        import time
+
+        @p.thunkify
+        def test(name, wait):
+            time.sleep(wait)
+            return f"{name}: {wait}"
+
+        async_test = test('a',2).asyncRun()
+        print(async_test.asyncWait()) #=> a: 2
+        ```
+        """
+        if not isinstance(fn, curry_fn):
+            self.exception('`fn` must be a thunk')
+        return fn.asyncRun()
+
+    def asyncWait(self, fn):
+        """
+        Function:
+
+        - Waits for a supplied function (if needed) and returns the results
+
+        Requires:
+
+        - `fn`:
+            - Type: function | method
+            - What: The function or method for which to wait
+            - Note: The supplied `fn` must have previously called `asyncRun`
+
+        Notes:
+
+        - A thunkified function that has called `asyncRun` can call `asyncWait` in place
+
+        Examples:
+
+        ```
+        import time
+
+        @p.thunkify
+        def test(name, wait):
+            time.sleep(wait)
+            return f"{name}: {wait}"
+
+        async_test = p.asyncRun(test('a',2))
+        print(p.asyncWait(async_test)) #=> a: 2
+        ```
+
+
+        ```
+        import time
+
+        @p.thunkify
+        def test(name, wait):
+            time.sleep(wait)
+            return f"{name}: {wait}"
+
+        async_test = p.asyncRun(test('a',2))
+        print(async_test.asyncWait()) #=> a: 2
+        ```
+        """
+        if not isinstance(fn, curry_fn):
+            self.exception('`fn` must be a thunk')
+        return fn.asyncWait()
 
     def clamp(self, minimum, maximum, a):
         """
@@ -350,12 +462,11 @@ class pamda_core(utils):
         """
         if isinstance(path, str):
             path=[path]
-        if not self.hasPath(data, path):
+        if not self.hasPath(path=path, data=data):
             self.warn(message="Path does not exist")
-            return data
-        if len(path)==0:
-            return {}
-        return self.assocPath(data=data, path=path[:-1], value={key:value for key, value in self.path(data, path=path[:-1]).items() if key!=path[-1]})
+        else:
+            reduce(self.getForceDict, path[:-1], data).pop(path[-1])
+        return data
 
     def flatten(self, data):
         """
@@ -647,12 +758,7 @@ class pamda_core(utils):
         """
         if isinstance(path, str):
             path=[path]
-        if len(path) > 0:
-            if path[0] in data:
-                return self.hasPath(data=data[path[0]], path=path[1:])
-            else:
-                return False
-        return True
+        return path[-1] in reduce(lambda x,y: x.get(y,{}), path[:-1], data)
 
     def hardRound(self, decimal_places, a):
         """
@@ -1030,7 +1136,7 @@ class pamda_core(utils):
         """
         Function:
 
-        - Returns the value of a path within a nested dictionary
+        - Returns the value of a path within a nested dictionary or None if the path does not exist
 
         Requires:
 
@@ -1049,11 +1155,7 @@ class pamda_core(utils):
         p.path(path=['a','b'], data=data) #=> 1
         ```
         """
-        if isinstance(path, str):
-            path=[path]
-        if len(path) > 0:
-            return self.path(data=data[path[0]], path=path[1:])
-        return data
+        return self.pathOr(None, path, data)
 
     def pathOr(self, default, path, data):
         """
@@ -1083,12 +1185,7 @@ class pamda_core(utils):
         """
         if isinstance(path, str):
             path=[path]
-        if len(path) > 0:
-            if path[0] in data:
-                return self.path(data=data[path[0]], path=path[1:])
-            else:
-                return default
-        return data
+        return reduce(lambda x, y: x.get(y, {}), path[:-1], data).get(path[-1], default)
 
     def pipe(self, fns, data):
         """
